#include "asm86.h"
#include "ti86asm.inc"

.org _asm_exec_ram

 call _clrLCD
 call _runindicoff
 ld a,32      ; Virtual screen xsize (32 columns = 256 pixels)
 ld hl,128    ; Virtual screen ysize (128 pixels)
 ld bc,$0000  ; Window left = 1 column, window top = 4
 ld de,$1040  ; Window hsize = 14 columns,  windows vsize = 50 rows
 ld (topleft),bc
 ld (size),de
 call SetupEngine
 ld hl,demoscr
 ld de,$8000  ; Virtual screen starts here
 ld bc,$1000
 ldir
 
 call UpdateScreen

MainLoop:
 ld a,$FE
 out (1),a
 in a,(1)
 ld b,a
 ld a,$BF
 out (1),a
 in a,(1)
 or $9F
 and b
 ld b,a
 ld a,$DF
 out (1),a
 in a,(1)
 or $7F
 and b
 ld b,a
 ld a,$EF
 out (1),a
 in a,(1)
 or $7F
 rrca
 rrca
 rrca
 and b
 bit 7,a
 jr z,MoveWin
 bit 4,a
 jr z,MoveWinPos
 ld b,1
 bit 5,a
 jr nz,RepLoop
 ld b,2
RepLoop:
 push bc
 bit 1,a
 push af
 call z,ScrollRight
 pop af
 bit 2,a
 push af
 call z,ScrollLeft
 pop af
 bit 0,a
 push af
 call z,ScrollUp
 pop af
 bit 3,a
 push af
 call z,ScrollDown
 pop af
 pop bc
 djnz RepLoop
 bit 6,a
 jr z,QuitDemo
 call UpdateScreen
 jr MainLoop

QuitDemo:
 call TrashEngine
 call _clrScrn
 ret

MoveWin:
 ld bc,(topleft)
 ld de,(size)
 bit 0,a
 push af
 call z,IncWinY
 pop af
 bit 3,a
 push af
 call z,DecWinY
 pop af
 bit 1,a
 push af
 call z,DecWinX
 pop af
 bit 2,a
 push af
 call z,IncWinX
 pop af
 ld (topleft),bc
 ld (size),de
 call ChangeWinSize
 call UpdateScreen
 jp MainLoop

MoveWinPos:
 ld bc,(topleft)
 ld de,(size)
 bit 0,a
 push af
 call z,IncWinPosY
 pop af
 bit 3,a
 push af
 call z,DecWinPosY
 pop af
 bit 1,a
 push af
 call z,DecWinPosX
 pop af
 bit 2,a
 push af
 call z,IncWinPosX
 pop af
 ld (topleft),bc
 ld (size),de
 call ChangeWinSize
 call UpdateScreen
 jp MainLoop

IncWinY:
 ld a,c
 add a,e
 cp 64
 ret z
 inc e
 push bc
 push de
 call ScrollDown
 pop de
 pop bc
 ret
IncWinX:
 ld a,b
 add a,d
 cp 16
 ret z
 inc d
 push bc
 push de
 ld b,8
IWX:
 push bc
 call ScrollRight
 pop bc
 djnz IWX
 pop de
 pop bc
 ret
DecWinY:
 ld a,e
 dec a
 ret z
 ld e,a
 add a,c
 call ClearRow
 ret
DecWinX:
 ld a,d
 dec a
 ret z
 ld d,a
 add a,b
 call ClearCol
 ret

IncWinPosY:
 ld a,c
 add a,e
 cp 64
 ret z
 ld a,c
 inc c
 call ClearRow
 ret
IncWinPosX:
 ld a,b
 add a,d
 cp 16
 ret z
 ld a,b
 inc b
 call ClearCol
 ret
DecWinPosY:
 ld a,c
 or a
 ret z
 dec c
 add a,e
 dec a
 call ClearRow
 ret
DecWinPosX:
 ld a,b
 or a
 ret z
 dec b
 add a,d
 dec a
 call ClearCol
 ret

ClearRow:
 push bc
 push de
 ld h,0
 ld l,a
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 ld de,$F800
 add hl,de
 bit 0,a
 ld a,$55
 jr z,EvenRow
 ld a,$AA
EvenRow:
 push hl
 push hl
 ld d,h
 ld e,l
 inc de
 ld bc,15
 ld (hl),a
 ldir
 pop de
 pop hl
 ld c,16
 set 2,d
 ldir
 pop de
 pop bc
 ret

ClearCol:
 push bc
 push de
 ld hl,$F800
 ld de,$FC00
 ld b,0
 ld c,a
 add hl,bc
 ex de,hl
 add hl,bc
 ld b,64
 ld a,$55
RepClearCol:
 push bc
 ld bc,16
 ld (hl),a
 add hl,bc
 ex de,hl
 ld (hl),a
 add hl,bc
 rlca
 pop bc
 djnz RepClearCol
 pop de
 pop bc
 ret

 #include "scroll.h"

demoscr:
 #include "scrdemo.db"

topleft: .dw 0
size:	   .dw 0

.end
