; GAME ENGINE
;
; by Jimmy Mrdell <yarin@acc.umu.se>
;
; You may use this code in your own game, improve on it, steal ideas from it, etc etc
;

Transfer	 = $B000
membackup  = $B100
newstack	 = $B500
rotTable	 = $B500
xSize 		 = $BD00
ySize 		 = $BD02
xWin			 = $BD04
yWin			 = $BD05
xWinSize	 = $BD06
yWinSize	 = $BD07
LCDofs		 = $BD08
LCDaddr 	 = $BD0A
LCDbyte 	 = $BD0C
xTilePos	 = $BD20
xBitPos 	 = $BD21
xPos			 = $BD22
yPos			 = $BD24
yOfs			 = $BD26
GetOffset  = $BD40

; SetupEngine:
;
; A = xsize/8, HL = ysize, B,C = top left of window, D,E = window size

SetupEngine:
 ld (ySize),hl
 ld hl,0
 ld (xSize),hl
 ld (xSize),a
 ld (xTilePos),hl
 ld (xPos),hl
 ld (yPos),hl
 ld (yOfs),hl
 call ChangeWinSize
 ld hl,rotTable+255
 ld b,0
InitRotLoop:
 dec b
 ld (hl),b
 dec hl
 jr nz,InitRotLoop
 ld hl,rotTable+256
 ld de,rotTable
 ld bc,$700
DoInitRotate:
 ld a,(de)
 inc de
 rlca
 ld (hl),a
 inc hl
 dec bc
 ld a,b
 or c
 jr nz,DoInitRotate
 ld hl,(xSize)
 dec hl
 ld (XSizeDec+1),hl
 ld de,(xSize)
 ld hl,GetOffset
 ld (hl),$6B
 inc hl
 ld (hl),$62
 inc hl
 xor a
 ld b,16
CreateGetOffset:
 or a
 jr z,NoAddHLHL
 ld (hl),$29
 inc hl
NoAddHLHL:
 sla e
 rl d
 jr nc,NoAddHLDE
 or a
 ld a,1
 jr z,NoAddHLDE
 ld (hl),$19
 inc hl
NoAddHLDE:
 djnz CreateGetOffset
 ld (hl),$C9

 di
 ld hl,$F800
 ld de,membackup
 ld bc,$400
 ldir
 ld hl,newstack-$FC00
 add hl,sp
 ld sp,hl
 ld a,$3C
 ld (LCDbyte),a
 ld hl,$FC00
 ld (LCDaddr),hl
 ld de,$F800
 ld bc,$400
 ldir
 ei
 ret

TrashEngine:
 di
 ld a,$3C
 out (0),a
 ld hl,membackup
 ld de,$F800
 ld bc,$400
 ldir
 ld hl,$FC00-newstack
 add hl,sp
 ld sp,hl
 ei
 ret

; ChangeWinSize
;
; B,C = top left of window, D,E = window size

ChangeWinSize:
 ld a,b
 ld (xWin),a
 ld a,c
 ld (yWin),a
 ld a,d
 ld (xWinSize),a
 ld a,e
 ld (yWinSize),a
 ld h,0
 ld l,c
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 ld c,b
 ld b,0
 add hl,bc
 ld (LCDofs),hl
 push de
 ld a,(xSize)
 sub d
 ld (MaxXTilePos+1),a
 ld d,0
 ld hl,(ySize)
 sbc hl,de
 ld (MaxYPos+1),hl
 pop bc
 ld hl,Transfer
 dec b
 jr z,EndRet
 ex de,hl
CopyCode:
 push bc
 ld hl,UnrolledCode
 ld bc,UnrolledCodeEnd-UnrolledCode
 ldir
 pop bc
 djnz CopyCode
 ex de,hl
EndRet:
 ld (hl),$C9
 ret

UpdateScreen:
 di
 ld a,(xBitPos)
 ld b,0
 ld c,a
 ld hl,MaskTbl
 add hl,bc
 ld d,(hl)
 ld a,d
 cpl
 ld e,a
 ld a,c
 ld hl,(yOfs)
 ld bc,$8000
 add hl,bc
 ld bc,(xTilePos)
 ld b,0
 add hl,bc
 push de
 add a,rotTable/$100
 ld d,a
 exx		 ; d' = hi(rottable)  hl' -> top left virt scr
 ld hl,(LCDofs)
 ld a,(LCDbyte)
 xor $04
 ld (LCDbyte),a
 add a,$C0
 ld d,a
 ld e,0
 add hl,de
 ld (LCDaddr),hl
 pop de  ; hl -> top left phys scr	d=lmask, e=rmask

 ld a,(yWinSize)
 ld b,a
TransferRow:
 push bc
 push hl
 exx
 push hl
 ld e,(hl)
 ld a,(de)
 exx
 call Transfer
 and d
 ld c,a
 exx
 inc hl
 ld e,(hl)
 pop hl
 ld bc,(xSize)
 add hl,bc
 ld a,(de)
 exx
 and e
 or c
 ld (hl),a
 pop hl
 ld bc,$10
 add hl,bc
 pop bc
 djnz TransferRow
 ld a,(LCDbyte)
 out (0),a
 ei
 ret

MaskTbl:
 .db $FF,$FE,$FC,$F8,$F0,$E0,$C0,$80

UnrolledCode:
 and d
 ld c,a
 exx
 inc hl
 ld e,(hl)
 ld a,(de)
 exx
 ld b,a
 and e
 or c
 ld (hl),a
 inc hl
 ld a,b
UnrolledCodeEnd:

ScrollRight:
 ld hl,(xPos)
 ld a,h
 or l
 ret z
 dec hl
 ld (xPos),hl
 ld hl,xBitPos
 dec (hl)
 ld a,(hl)
 cp -1
 ret nz
 ld (hl),7
 ld hl,xTilePos
 dec (hl)
 ret

ScrollLeft:
 ld hl,xTilePos
MaxXTilePos:
 ld a,0
 cp (hl)
 ret z
 ex de,hl
 ld hl,(xPos)
 inc hl
 ld (xPos),hl
 ld hl,xBitPos
 ld a,(hl)
 inc a
 and 7
 ld (hl),a
 ret nz
 ex de,hl
 inc (hl)
 ret

ScrollDown:
 ld hl,(yPos)
 ld a,h
 or l
 ret z
 dec hl
 ld (yPos),hl
 ld hl,(yOfs)
 ld de,(xSize)
 sbc hl,de
 ld (yOfs),hl
 ret

ScrollUp:
 ld hl,(yPos)
MaxYPos:
 ld de,0
 call $403C ; _cp_hl_de
 ret z
 inc hl
 ld (yPos),hl
 ld hl,(xSize)
 ld de,(yOfs)
 add hl,de
 ld (yOfs),hl
 ret

; Sprite putting routines (using XOR)
; 3600 8x8 sprites/second, 2100 8x16 sprites/second

Put8Sprite:
 ld a,8
 jr PutSprite
Put16Sprite:
 ld a,16
PutSprite:	 ; A = lines, BC,DE = x,y, HL -> sprite
 di
 push af
 push hl
 call FindPixel
 exx
 ld b,0
 ld c,a
 ld hl,MaskTbl
 add hl,bc
 add a,rotTable/$100
 ld d,a
 ld a,(hl)
 pop hl
 pop bc
 exx
 ld d,a
 cpl
 ld e,a
 exx
PS_Line:	; B'=lines, D/E=masks, HL->phys scr, DE'->rottable, HL'->sprite
 ld e,(hl)
 inc hl
 ld a,(de)
 exx
 ld c,a
 and e
 xor (hl)
 ld (hl),a
 inc hl
 ld a,c
 and d
 xor (hl)
 ld (hl),a
XSizeDec:
 ld bc,0
 add hl,bc
 exx
 djnz PS_Line
 exx
 ret

; Input: BC,DE = x,y  Output: HL->virtual screen, A=mask

FindPixel:
 call GetOffset
 ld de,$8000
 add hl,de
 ld a,7
 sub c
 and 7
 srl b
 rr c
 srl b
 rr c
 srl b
 rr c
 add hl,bc
 ret
